open GMain
open Couleurs
open Data
open Figure

(************fonctions de base pour les hexagones**************************)

(*fabrique une liste de six points correspondant aux sommets de l'hexagone*)
(* sachant que (x,y) correspond au coin sup gauche *)

let make_six x y hauteur long = 
  [(int_of_float(x),int_of_float(y));
   (int_of_float(x+.hauteur),int_of_float(y-.(long/.2.0)));
   (int_of_float(x+.(2.0*.hauteur)),int_of_float(y));
   (int_of_float(x+.(2.0*.hauteur)),int_of_float(y+.long));
   (int_of_float(x+.hauteur),int_of_float(y+.(3.0*.long/.2.0)));
   (int_of_float(x),int_of_float(y+.long))]
    
let make_six2 x y hauteur long = 
  [ (x,y); 
    (x+hauteur, y-long/2);
    (x+2*hauteur, y);
    (x+2*hauteur, y+long);
    (x+hauteur, y+ 3 * long / 2 );
    (x, y+long) ] ;;
    
(*fonction qui trace un hexagone*)

(*x,y : abs et ord du coin sup gauche de l'hexagone*)
(*long : unite, c'est a dire longueur d'un cote de polygone*)

(**************************************************************************)
(*draw_hexa :                                                             *)
(**************************************************************************)
let draw_hexa (x:float) (y:float) (long:float) 
    (drawing: 'a GDraw.drawable) = 
  let  hauteur =  (sqrt(3.0))*.long/.2.0 in
    drawing#polygon  ~filled:false  (make_six x y hauteur long)
;;




let draw_line (x:int) (y:int) 
    (long:float) 
    (dx:int) (dy:int) 
    (drawing: 'a GDraw.drawable) 
    (c:i_couleur ) = 
 
  let h =   (sqrt(3.0))*.long/.2.0  in
  let hauteur = int_of_float h
  and demi_hauteur = int_of_float (h /. 2.0) in
  let x' = x+dx + hauteur 
  and y' = y+dy + demi_hauteur 
  in                                          
    drawing#set_foreground (`RGB c); (*On lui donne sa couleur*)
    drawing#line 
      (x+ hauteur) 
      (y+ demi_hauteur) x' y'  
;;


(***fonctions pour afficher le maillage hexagonal du premier type*********)

(*************************************************************************)
(*ligne_hexa :                                                           *)
(*************************************************************************)
    
(* fonction qui trace une ligne de larg hexas vides*)
let ligne_hexa  (x:float) (y:float) (long:float) (larg:int)
    (drawing: 'a GDraw.drawable)  =
  let  ht =  (sqrt(3.0))*.long/.2.0 in
  let rec trace_lig longueur largeur = function
      (abs,ord,lig) when (lig=largeur) -> (draw_hexa abs ord longueur drawing)
    |(abs,ord,lig) ->  (draw_hexa abs ord longueur drawing);
        (trace_lig longueur largeur ((abs+.2.0*.ht),ord,lig+1))
  in
    (trace_lig long larg (x,y,1));; 


(************************************************************************)
(*draw_maillage_hexa1 :                                                 *)
(************************************************************************) 

(*fonction qui affiche le maillage hexagonal de premier type*)
let draw_maillage_hexa1  (x:float) (y:float) (long:float) (larg:int) (drawing: 'a GDraw.drawable) =
     
     let  ht =  (sqrt(3.0))*.long/.2.0 in
     let rec trace_tt longueur largeur = function 
       (abs,ord,ligne) when (ligne=largeur) -> 
	 (ligne_hexa abs ord longueur largeur drawing)
      |(abs,ord,ligne) -> 
	 (ligne_hexa abs ord longueur largeur drawing);
          if ((ligne mod 2)=0) then                           
            (trace_tt longueur largeur (0.0,ord+.(3.0*.longueur/.2.0),ligne+1))
          else                            
            (trace_tt longueur largeur (ht,ord+.(3.0*.longueur/.2.0),ligne+1))
     in
       (trace_tt long larg (x,y,1));;



(*****fonctions pour afficher le maillage hexagonal du deuxieme type******)

(*************************************************************************)
(*draw_maillage_hexa2 :                                                  *)
(*************************************************************************)

(*fonction qui affiche le maillage hexagonal de deuxieme type*)
let draw_maillage_hexa2  (x:float) (y:float) (long:float) (larg:int) (drawing: 'a GDraw.drawable) =
     
     let  ht =  (sqrt(3.00))*.long/.2.00 in
     let rec trace_tt longueur largeur = function 
       (abs,ord,ligne) when (ligne=largeur) -> 
	 (ligne_hexa abs ord longueur largeur drawing)
      |(abs,ord,ligne) -> 
	 (ligne_hexa abs ord longueur largeur drawing);                   
          (trace_tt longueur largeur (abs+.ht,ord+.(3.00*.longueur/.2.00),ligne+1))
     in
       (trace_tt long larg (x,y,1));;


(******************************************************************)
(*****fonctions de remplissage****************************************)
(*************************************************************************)

let hauteur_of_cote c = 
  let uni' = float_of_int c in
  let hauteur=(sqrt(3.00)) *. uni' /. 2.0 
  in int_of_float hauteur

and demi_hauteur_of_cote c = 
  let uni' = float_of_int c in
  let hauteur=(sqrt(3.00)) *. uni' /. 4.0 
  in int_of_float hauteur;;




 let figure_hexa chan unite c x_min y_min d =  
 
    if not (Data.exists_champ chan d)
    then None
    else
      let  h=(sqrt(3.00)) *. (float_of_int unite) /. 2.0
      in let  hauteur =  int_of_float h
	 and demi_hauteur = int_of_float (h /. 2.0)
      	 and valeur =(Data.get_champ chan d)
	 and x,y = Data.get_pos  d
	  
      in
	( (* coordonnes en pixels dans le plan de l'image *)
	  let abs=x*2*hauteur + y*hauteur 
	    (* 1x dans les coordonnes GBF = 2hx dans le plan image *)
	    (* 1y dans les coordonnes GBF => hy dans le plan image *)
	    
          and ord=y*3/2*unite + 1/2*unite
	    (* 1y dans les coordonnes GBF => (3/2)*cote*y dans le plan image *)
	  
	  
	in
	    match valeur with 
		
	      | Vect (dx,dy) -> (* 12 *)
		  let triplet=  int_of_float3 (mult_couleur (255.0*.255.0) c)
		  in
		  let x'= abs + hauteur 
		  and y'= ord+demi_hauteur
                  in
		    Some
		      (Ligne ( (x', y'), (x'+ dx*unite, y'+dy*unite), triplet))  
		      
	      | Int v ->
		  let triplet=
		    int_of_float3 (mult_couleur (float_of_int v *. 255.0 ) c )
		  in Some (Hexagone ( (abs,ord), unite, triplet) )	
	)


(** calcule toutes les figures  afficher pour un champ dans une scene *)
(** @param xmin attention : non utilise dans le mode hexa *)
let figures  ch unite c xmin ymin (s:Types.scene) = 
  let f = (fun e -> fun r -> 
    match (figure_hexa ch unite c xmin ymin e) with 
      None -> r 
    | Some f -> f :: r)
  in List.fold_right f  s []



	     
       
