/* parser.mly */
%{
open Types;;
open Data;;
exception SyntaxError of string ;;
exception NotAvailable of string ;;

let data_of_list l =
  let px = match List.assoc "posx" l with Int i -> i | _ -> raise (SyntaxError "posx must be an integer)")
  and py = match List.assoc "posy" l with Int i -> i | _ -> raise (SyntaxError "posx must be an integer)")
  and l' = List.remove_assoc "posx" (List.remove_assoc "posy" l)
  in Data.make_data (px, py) l'  
    

%}

%token <string> STRING
%token <int> INT
%token  LACCOL  RACCOL
%token  LCROCH  RCROCH
%token  VIRG  EGAL
%token  GRILLE HEXA 
%token  COORD 
%token  MESS 
  
%token EOF
%token MAX

%right EGAL
%nonassoc LACCOL RACCOL
%nonassoc MAX0

%start main entete scene
%type <(string option)*Types.maille*(Data.data list)> main
%type <(string option)*Types.maille> entete
%type <Types.scene> scene

%%

entete : 
            topologie reperage { (None, $1) } 
  | message topologie reperage { (Some $1 , $2) } ;
  |         topologie  { (None, $1) } 
  | message topologie  { (Some $1 , $2) } ;

message : MESS STRING { $2 } ;

topologie : 
    GRILLE { Grille }
  | HEXA   { Hexa   }
  | error  { raise (SyntaxError "topology") }
;

reperage :
    INT      { raise (NotAvailable "Sequential coordinates are not implemented anymore (please provide coordinates to your data).") }
  | COORD    { } /* to keep compatibility with files created for v0.1 to v0.4 */


;

main :
  entete  scene { match $1 with (m,t) -> (m, t, $2) }


seq :

  | scene  seq %prec MAX { $1::$2}
  | scene  { $1::[]}
  | error  { raise (SyntaxError "scene") }

;


scene : 
  LCROCH data_list RCROCH { $2}
  | EOF { raise Types.End_Of_File}
;

data_list : 
     LACCOL record RACCOL data_list %prec MAX { (data_of_list $2)::$4 }
   | LACCOL record RACCOL { (data_of_list $2)::[] }
   | error  { raise (SyntaxError "data list") }
;

record :

   | STRING EGAL value VIRG record  %prec MAX{ ($1,$3):: $5 }
   | STRING EGAL value {($1,$3) :: [] } 
   | error  { raise (SyntaxError "record") }
;
   

vector :
   | LACCOL STRING EGAL INT VIRG STRING EGAL INT RACCOL {match ($2,$6) with
							   |  ("vx","vy") -> ($4,$8)
							   | ("vy","vx") -> ($8,$4)
							   | _ -> raise (SyntaxError "vector is restricted to vx and vy") }
   | error  { raise (SyntaxError "vector") }

value : 
   | INT { Int $1 }
   | vector { Vect $1 }
   | error  { raise (SyntaxError "value") }
;

